#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
#
#   file: t/ErrorLoggerTestPlugin.pm
#
#   This file is part of perl-Dist-Zilla-Role-ErrorLogger.
#
#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

#   This is a trivial plugin which allows test to execute arbitrary code. Test should save
#   reference to code to be executed in $ErrorLoggerTestPlugin::Hook variable.

package ErrorLoggerTestPlugin;

use Moose;
use namespace::autoclean;

with 'Dist::Zilla::Role::FileGatherer';
with 'Dist::Zilla::Role::ErrorLogger';

our $Hook;

sub gather_files {
    my ( $self ) = @_;
    $Hook->( $self );
    return;
};

__PACKAGE__->meta->make_immutable;

1;

# end of file #
